package com.ctpodp;

import com.jibestream.jmapandroidsdk.astar.ASNode;
import com.jibestream.jmapandroidsdk.rendering_engine.moving_objects.MovingObject;

import android.graphics.PointF;

/**
 * Created by Andrew Adams on 2017-05-02.
 */

public class ArrowDrawable extends MovingObject {

  public ArrowDrawable(float x, float y) {
    super(x, y);
  }

  public void followPath(final ASNode[] animationPoints, final DestinationReachedCallback destinationReached) {
    new Thread(new Runnable() {
      @Override
      public void run() {
        animationThroughPoints(animationPoints, 1);
      }

      private void animationThroughPoints(final ASNode[] animationPoints, final int index) {
        // Create points, calculate angle between the points and set rotation for drawable
        PointF current = new PointF((float) animationPoints[index - 1].x, (float) animationPoints[index - 1].y);
        PointF target = new PointF((float) animationPoints[index].x, (float) animationPoints[index].y);
        setRotation(getAngleBetweenPoints(current, target));

        // Animate the drawable
        animateTo(new PointF((float) animationPoints[index].x, (float) animationPoints[index].y), new MovingObjectCallback() {
          @Override
          public void onLocationChanged(PointF pointF) {

          }

          @Override
          public void onAnimationComplete(PointF pointF) {
            // If we haven't animated to all points, continue animating
            if (index + 1 < animationPoints.length) {
              int newindex = index + 1;
              animationThroughPoints(animationPoints, newindex);
            } else if (index + 1 >= animationPoints.length) { // Otherwise send the callback
              destinationReached.onDestinationReached();
            }
          }
        });
      }

      private float getAngleBetweenPoints(PointF centerPt, PointF targetPt) {
        double theta = Math.atan2(targetPt.y - centerPt.y, targetPt.x - centerPt.x) + Math.PI / 2.0;
        double angle = Math.toDegrees(theta);

        if (angle < 0) {
          angle += 360;
        }

        return (float) angle;
      }

    }).start();
  }

  public interface DestinationReachedCallback {
    void onDestinationReached();
  }
}
