package com.ctpodp;

import static com.facebook.react.bridge.UiThreadUtil.runOnUiThread;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.PointF;
import android.os.Bundle;

import androidx.fragment.app.Fragment;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.ctpodp.objects.MapTapEvent;
import com.ctpodp.objects.NavigationPathGenerated;
import com.ctpodp.objects.PathAndWaypoints;
import com.ctpodp.objects.ProcessedInstruction;
import com.ctpodp.objects.fromReact.CenterMapOnUserObj;
import com.ctpodp.objects.fromReact.DrawNavigationPathObj;
import com.ctpodp.objects.fromReact.MapCredentials;
import com.ctpodp.objects.fromReact.SetFollowBlueDotObj;
import com.ctpodp.objects.fromReact.SetMapVenueObj;
import com.ctpodp.objects.fromReact.SetUserPositionObj;
import com.ctpodp.objects.fromReact.WayfindBetweenWaypointsObj;
import com.ctpodp.objects.fromReact.WayfindToWaypointObj;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.facebook.react.uimanager.ThemedReactContext;
import com.jibestream.jmapandroidsdk.astar.PathPerFloor;
import com.jibestream.jmapandroidsdk.components.BaseModel;
import com.jibestream.jmapandroidsdk.components.Destination;
import com.jibestream.jmapandroidsdk.components.Waypoint;
import com.jibestream.jmapandroidsdk.jcontroller.JController;
import com.jibestream.jmapandroidsdk.jcore.JCore;
import com.jibestream.jmapandroidsdk.jmap.JMap;
import com.jibestream.jmapandroidsdk.jmap.JMapOptions;
import com.jibestream.jmapandroidsdk.main.PointD;
import com.jibestream.jmapandroidsdk.main.Utilities;
import com.jibestream.jmapandroidsdk.oauth.JAuth;
import com.jibestream.jmapandroidsdk.rendering_engine.MapLayer;
import com.jibestream.jmapandroidsdk.rendering_engine.MapView;
import com.jibestream.jmapandroidsdk.rendering_engine.Stage;
import com.jibestream.jmapandroidsdk.rendering_engine.Transform;
import com.jibestream.jmapandroidsdk.rendering_engine.drawables.JShapeDrawable;
import com.jibestream.jmapandroidsdk.rendering_engine.moving_objects.UserLocation;
import com.jibestream.jmapandroidsdk.styles.JStyle;
import com.jibestream.mapuikit.Compass;
import com.jibestream.mapuikit.CompassOptions;
import com.jibestream.mapuikit.FloorSelector;
import com.jibestream.mapuikit.FloorSelectorOptions;
import com.jibestream.mapuikit.MapUiKit;
import com.jibestream.mapuikit.Search;
import com.jibestream.mapuikit.SearchOptions;
import com.jibestream.navigationkit.NavigationKit;
import com.jibestream.navigationkit.instructionfactory.Instruction;

import java.util.Arrays;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Set;

public class MapFragment extends Fragment {

  ThemedReactContext _context;
  Stage stage;
  ProgressBar progressBar;
  JController controller;
  NavigationKit navigationKit;
  boolean isNavigating = false;
  Dictionary<String, PathPerFloor[]> paths = new Hashtable<String, PathPerFloor[]>();
  private JShapeDrawable[] highlightedUnit;
  private PopupView popupView;
  private SetFollowBlueDotObj followBlueDot = new SetFollowBlueDotObj(false, false, 500);
  private long lastBlueDotFollow = 0;
  private int userMaxSpeed = 50;

  public MapFragment(ThemedReactContext context) {
    _context = context;
    // Required empty public constructor
  }

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
  }

  @Override
  public View onCreateView(LayoutInflater inflater, ViewGroup container,
                           Bundle savedInstanceState) {
    // Inflate the layout for this fragment
    View view = inflater.inflate(R.layout.fragment_map, container, false);
    return view;
  }

  @Override
  public void onViewCreated(View view, Bundle savedInstanceState) {
//    _context = this.getContext();
    stage = view.findViewById(R.id.stage);
    progressBar = view.findViewById(R.id.progressBar);
//    initializeMap("Neaera Consulting - Fort Collins");
    Toast.makeText(_context, "View Created", Toast.LENGTH_SHORT).show();
//    new OdpModule(_context);
  }

  public void sendEvent(String topic, WritableMap data) {
    Log.d("EVENT", "Sending Event To " + topic + ", " + data.toString());
    _context.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
      .emit(topic, data);
  }

  public void sendEvent(String topic, WritableArray data) {
    Log.d("EVENT", "Sending Event To " + topic + ", " + data.toString());
    _context.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
      .emit(topic, data);
  }

  public void initializeMap(SetMapVenueObj obj) {
    MapCredentials credentials = obj.credentials;

    String CLIENT_ID = credentials.clientId;
    String CLIENT_SECRET = credentials.clientSecret;
    int CUSTOMER_ID = credentials.customerId;
    int VENUE_ID = credentials.venueId;
    String HOST = credentials.host;

    JAuth jAuth = new JAuth(CLIENT_ID, CLIENT_SECRET);

    JMapOptions mapOptions = new JMapOptions(jAuth, CUSTOMER_ID, VENUE_ID, HOST, stage);

    mapOptions.shouldParseAllMaps(true);
    mapOptions.shouldApplyDisplayMode(true);

    final JMap map = new JMap(_context, mapOptions);

    UserLocation.getInstance().snapToWayfindingPathWithThreshold(50);
//            UserLocation.getInstance().autoRerouteWayfindingPathWithThreshold(getWayfindingRerouteThreshold());
    UserLocation.getInstance().setSpeed(userMaxSpeed);
    UserLocation.getInstance().setDelayBetweenPulseCycles(700);
    UserLocation.getInstance().setPulseAnimationDuration(1500);

    // https://support.cxapp.com/hc/en-us/articles/360016554032-Configuring-Styling-the-User-Location-Indicator-example-

    Bitmap arrowBitmap = BitmapFactory.decodeResource(getResources(), R.drawable.path_arrow);
    arrowBitmap = Bitmap.createScaledBitmap(arrowBitmap, 25, 25, true);

    UserLocation.getInstance().setBitmap(arrowBitmap);

    map.init(
      new JMap.OnMapInitCallback() {
        @Override
        public void onReady() {
          runOnUiThread(new Runnable() {
            @Override
            public void run() {
              progressBar.setVisibility(View.GONE);
            }
          });

          controller = map.getController();

          controller.setOnTapListener(new MapView.OnTapListener() {
            @Override
            public void onTap(MotionEvent motionEvent, Object drawable) {
              MapTapEvent tapEvent = new MapTapEvent(motionEvent.getX(), motionEvent.getY());
              sendEvent("MAP_TAPPED_GENERIC", ReactUtilities.classToMap(tapEvent));
              if (drawable == null) {
                String[] mapLayers = controller.getAllHiddenLayerNamesInMap(controller.getCurrentMap());
                if (isNavigating) {
                  //set user location
                  //Utilities.convertEpsg4326ToLocal(convertPoint, controller.getActiveVenue());
                  PointF tappedLocation = new PointF(motionEvent.getX(), motionEvent.getY());
                  controller.updateUserLocationPosition(tappedLocation, 0, 100, controller.getCurrentMap());
                }
              }
//              else if (drawable instanceof JShapeDrawable) {
//                  //reset previously highlighted unit and popup
//                  controller.resetShapeStyles(highlightedUnit);
//                  controller.removeComponent(popupView, controller.getCurrentMap());
//
//                  //cast shape drawable
//                  JShapeDrawable shapeDrawable = (JShapeDrawable) drawable;
//
//                  //initialize new highlighted unit
//                  highlightedUnit = new JShapeDrawable[]{shapeDrawable};
//
//                  //create a highlight style
//                  JStyle highlightStyle = new JStyle();
//                  highlightStyle.setColor(Color.parseColor("#CDDC39"));
//                  highlightStyle.setStrokeColor(Color.parseColor("#202020"));
//                  highlightStyle.setStrokeWidth(1);
//
//                  //style unit
//                  controller.styleShapes(highlightedUnit, highlightStyle);
//
//                  //extract waypoint meta data from unit shape
//                  Object[] destinationIds = shapeDrawable.getMetaData("data-unit");
//
//                  //extract destination name
//                  String destinationName = "n/a";
//                  if (destinationIds != null && destinationIds.length > 0) {
//                    //get first destination from ids
//                    Destination destination = controller.getActiveVenue().getDestinations().getById((int) destinationIds[0]);
//
//                    //check if destination name exists
//                    if (destination.getName() != null) {
//                      destinationName = destination.getName();
//                    }
//                  }
//
//                  //initialize custom popup view
//                  popupView = new PopupView(_context);
//                  popupView.getPopupTextView().setText(destinationName);
//
//                  //add popup to map view
//                  controller.addComponent(popupView, controller.getCurrentMap(), new PointF(motionEvent.getX(), motionEvent.getY()));
//                }
              }
          });

//          controller.enableLayerInteractivity(MapLayer.Layer_Units);

          navigationKit = new NavigationKit(controller);

          final PointF centerOfMap = new PointF(1000, 1000); //controller.getCurrentMap().getWidth()/2, controller.getCurrentMap().getHeight()/2
          UserLocation.getInstance().setPosition(centerOfMap);

          WritableArray searchDestinations = ReactUtilities.classToArray(controller.getActiveVenue().getDestinations().getAll());
          Log.d("SEARCH DESTINATIONS", searchDestinations.toString());
          sendEvent("GENERATED_SEARCH_DESTINATIONS", searchDestinations);

          Waypoint userLocationWaypoint = controller.getActiveVenue().getClosestWaypointToCoordinatesOnMap(centerOfMap, controller.getCurrentMap());

          controller.addComponent(UserLocation.getInstance(), controller.getCurrentMap(), new PointF(userLocationWaypoint.getCoordinates()[0].floatValue(), userLocationWaypoint.getCoordinates()[1].floatValue()));
        }

        @Override
        public void onError(String s) {
          runOnUiThread(new Runnable() {
            @Override
            public void run() {
              progressBar.setVisibility(View.GONE);
            }
          });
        }
      }
    );
  }

  private float getAngleBetweenPoints(PointF centerPt, PointF targetPt) {
    double theta = Math.atan2(targetPt.y - centerPt.y, targetPt.x - centerPt.x) + Math.PI / 2.0;
    double angle = Math.toDegrees(theta);

    if (angle < 0) {
      angle += 360;
    }

    return (float) angle;
  }

  private float getDistanceBetweenPoints(PointF startPoint, PointF endPoint) {

    return (float) Math.sqrt(Math.pow(startPoint.x - endPoint.x,2) + Math.pow(startPoint.y - endPoint.y,2));
  }

  public void setUserPosition(SetUserPositionObj obj) {
    PointF newUserLocation = null;
    if (obj.format == null) {
      PointD convertPoint = new PointD(obj.y, obj.x);
      Utilities.convertEpsg4326ToLocal(convertPoint, controller.getActiveVenue());
      newUserLocation = new PointF((float) convertPoint.x, (float) convertPoint.y);
    }
    else if (obj.format.equals("xy")) {
      newUserLocation = new PointF((float) obj.x, (float) obj.y);
    }
    float heading = getAngleBetweenPoints(UserLocation.getInstance().getPosition(), newUserLocation);
    float distance = getDistanceBetweenPoints(UserLocation.getInstance().getPosition(), newUserLocation);
    controller.updateUserLocationPosition(newUserLocation, (int) heading, (int) obj.confidence, controller.getCurrentMap());

    if (followBlueDot.enable) {
      long currTime = System.currentTimeMillis();
      double rotate_heading = 0;
      int animationDuration = (int) (1000 * distance / userMaxSpeed);
      if (followBlueDot.rotate) rotate_heading = -heading;
      if (currTime - lastBlueDotFollow > followBlueDot.interval) {
        centerMapOnPosition(newUserLocation.x, newUserLocation.y, 5, rotate_heading, animationDuration);
      }
    }

    WritableArray data = Arguments.createArray();
    data.pushDouble(newUserLocation.x);
    data.pushDouble(newUserLocation.y);
    sendEvent("USER_POSITION_UPDATED", data);
  }

  public void centerMapOnPosition(double x, double y, double zoom, double orientation, int animationDuration) {
    controller.setMapTransform(new Transform((float) zoom, new PointF((float) x, (float) y), (float)orientation), animationDuration, new MapView.OnAnimationCallback() {
      @Override
      public void onAnimationComplete() {

      }

      @Override
      public void onAnimationInterrupted() {

      }
    });
  }

  public void centerMapOnUser(CenterMapOnUserObj obj) {
    double orientation = obj.orientation;
    int animationDuration = obj.animationDuration;
    int zoom = obj.zoom;
    Log.d("Command", "Centering On Map, " + orientation + ", " + animationDuration);
    controller.setMapTransform(new Transform(5, UserLocation.getInstance().getPosition(), (float)orientation), animationDuration, new MapView.OnAnimationCallback() {
      @Override
      public void onAnimationComplete() {

      }

      @Override
      public void onAnimationInterrupted() {

      }
    });
  }

  private void highlightUnit(Destination destination) {
    JStyle highlightStyle = new JStyle();
    highlightStyle.setColor(Color.parseColor("#CDDC39"));
    highlightStyle.setStrokeColor(Color.parseColor("#202020"));
    highlightStyle.setStrokeWidth(1);

    JShapeDrawable[] units = controller.getUnitsFromDestination(destination);
    controller.styleShapes(units, highlightStyle);
  }

  private void focusToWaypoint(Waypoint userLocationWaypoint) {
    Double[] waypointCoords = userLocationWaypoint.getCoordinates();
    controller.setMapTransform(new Transform(5, new PointF(waypointCoords[0].floatValue(), waypointCoords[1].floatValue()), 0), 1500, new MapView.OnAnimationCallback() {
      @Override
      public void onAnimationComplete() {

      }

      @Override
      public void onAnimationInterrupted() {

      }
    });
  }

  public void setFollowBlueDot(SetFollowBlueDotObj obj) {
    followBlueDot = obj;
    centerMapOnUser(new CenterMapOnUserObj(UserLocation.getInstance().getRotation(), 5, 1500));
  }

  public void clearWayfindingPath() {
    controller.clearWayfindingPath();
  }

  public void drawWayfindingPath(DrawNavigationPathObj obj) {
    String pathId = obj.pathId;
    Boolean shouldFocus = obj.shouldFocus;
    JStyle style = obj.style;

    clearWayfindingPath();
    PathPerFloor[] path = paths.get(pathId);
    controller.drawWayfindingPath(path, style);
    if (shouldFocus) {
      // TODO: set orientation based on first instruction direction
      centerMapOnUser(new CenterMapOnUserObj(UserLocation.getInstance().getRotation(), 5, 1500));
    }
  }

  public void wayfindToWaypoint(WayfindToWaypointObj obj) {
    int waypointId = obj.waypointId;
    PathAndWaypoints pathAndWaypoints = getPathToDestination(waypointId);
    Instruction[] instructions = navigationKit.createInstructionsFromPaths(pathAndWaypoints.path);
    ProcessedInstruction[] processedInstructions = processInstructions(instructions);
    String path_id = pathAndWaypoints.origin.id + "_" + pathAndWaypoints.destination.id;
    paths.put(path_id, pathAndWaypoints.path);
    sendEvent("NAVIGATION_PATH_GENERATED", ReactUtilities.classToMap(new NavigationPathGenerated(path_id, processedInstructions, pathAndWaypoints.origin, pathAndWaypoints.destination, pathAndWaypoints.path)));
  }

  public void wayfindBetweenWaypoints(WayfindBetweenWaypointsObj obj) {
    int originId = obj.originId;
    int destinationId = obj.destinationId;
    PathAndWaypoints pathAndWaypoints = getPathBetweenWaypoints(originId, destinationId);
    Instruction[] instructions = navigationKit.createInstructionsFromPaths(pathAndWaypoints.path);
    ProcessedInstruction[] processedInstructions = processInstructions(instructions);
    String path_id = pathAndWaypoints.origin.id + "_" + pathAndWaypoints.destination.id;
    paths.put(path_id, pathAndWaypoints.path);
    sendEvent("NAVIGATION_PATH_GENERATED", ReactUtilities.classToMap(new NavigationPathGenerated(path_id, processedInstructions, pathAndWaypoints.origin, pathAndWaypoints.destination, pathAndWaypoints.path)));
  }

  private PathAndWaypoints getPathToDestination(int destinationId) {
    Waypoint userLocationWaypoint = controller.getActiveVenue().getClosestWaypointToCoordinatesOnMap(UserLocation.getInstance().getPosition(), controller.getCurrentMap());
    return getPathBetweenWaypoints(userLocationWaypoint.id, destinationId);
  }

  private PathAndWaypoints getPathBetweenWaypoints(int sourceId, int destinationId) {
    Waypoint sourceWaypoint = Arrays.stream(controller.getActiveVenue().getMaps().getAllWaypoints()).filter(wp -> wp.id == sourceId).findFirst().get();
    Waypoint destinationWaypoint = Arrays.stream(controller.getActiveVenue().getMaps().getAllWaypoints()).filter(wp -> wp.id == destinationId).findFirst().get();
    PathPerFloor[] pathPerFloors = controller.wayfindBetweenWaypoints(sourceWaypoint, destinationWaypoint);
    return new PathAndWaypoints(sourceWaypoint, destinationWaypoint, pathPerFloors);

//                    currentInstructions = processInstructions(navigationKit.createInstructionsFromPaths(pathPerFloors));
//                    onLocationChange(UserLocation.getInstance().getPosition());
  }


  private ProcessedInstruction[] processInstructions(Instruction[] instructions) {
    return Arrays.stream(instructions).map((Instruction instruction) -> {
      ProcessedInstruction processedInstruction = new ProcessedInstruction();
      if (instruction.getDecisionPoint() != null) {
        processedInstruction.wp = instruction.getDecisionPoint().id;
      }
      else {
        processedInstruction.wp = instruction.getCompletionPoint().id;
      }
      Float distance = instruction.getPixelDistance() * controller.getCurrentMap().getMmPerPixel() / 1000;
      String units = " meters";
      if (Math.round(distance) == 1) units = " meter";
      processedInstruction.distance = Double.valueOf(distance);
      processedInstruction.remainingDistance = Double.valueOf(distance);
      processedInstruction.direction = instruction.getDirection();

      String prepend = "";
      if (instruction.getDirection().equals("Left") || instruction.getDirection().equals("Right")) {
        prepend = "Turn ";
      }
      processedInstruction.prepend = prepend;

      if (instruction.getSurroundingElements() != null &&
        instruction.getSurroundingElements().getShapes() != null &&
        instruction.getSurroundingElements().getShapes().length > 0) {
        JShapeDrawable shape = instruction.getSurroundingElements().getShapes()[0].getShape();

        //Get data-unit meta data which is associated to units
        Object[] dataUnits = shape.getMetaData("data-unit");

        //Make sure shape has destination association
        if (dataUnits != null && dataUnits.length > 0) {
          int destinationId = (int) dataUnits[0];

          //Get destination object from Id
          Destination destination = controller.getActiveVenue().getDestinations().getById(destinationId);

          //Get destination name
          String destinationName = destination.getName();

          if (destinationName != null) {
            processedInstruction.destinationName = destinationName;

            processedInstruction.dest = destination.id;
            processedInstruction.fullText = instruction.getText() + " at " + destinationName + " in " + Math.round(distance) + units;

          }
          else {
            processedInstruction.dest = destination.id;
            processedInstruction.fullText = instruction.getText() + " in " + Math.round(distance) + units;
          }
        }
        else {
          processedInstruction.destinationName = "";
          processedInstruction.dest = null;
          processedInstruction.fullText = instruction.getText() + " in " + Math.round(distance) + units;
        }
      }
      return processedInstruction;
    }).toArray(ProcessedInstruction[]::new);
  }

  @Override
  public void onResume() {
    super.onResume();
    stage.getMapView().resumeMap();
  }

  @Override
  public void onPause() {
    super.onPause();
    try {
      stage.getMapView().pauseMap();
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
  }
}
