import React, { useImperativeHandle, useEffect, useRef, forwardRef, useState } from 'react';
import { SafeAreaView, findNodeHandle,
// PixelRatio,
DeviceEventEmitter, NativeEventEmitter, Platform, PixelRatio } from 'react-native';
import { MapViewManager, CTPEventEmitter } from './MapViewManager';
import { centerMap, createFragment, drawNavigationPath, setFollowBlueDot, setMapVenue, setUserPosition, clearWayfindingPath, wayfindToWaypoint, wayfindBetweenWaypoints } from './SendCommand';
const MapView = (props, parentRef) => {
  const nodeRef = useRef(null);
  const [onNavigationPathGeneratedValue, setOnNavigationPathGeneratedValue] = useState(null);
  const [onDestinationSelectedValue, setOnDestinationSelectedValue] = useState(null);
  const [onUserPositionUpdatedValue, setOnUserPositionUpdatedValue] = useState(null);
  const [onSearchDestinationsGeneratedValue, setOnSearchDestinationsGeneratedValue] = useState(null);
  const [onMapTappedGenericValue, setOnMapTappedGenericValue] = useState(null);
  useEffect(() => {
    if (Platform.OS == 'android') {
      createFragment(findNodeHandle(nodeRef.current));
    }
  }, []);
  useEffect(() => {
    var _props$onMapMessage$o, _props$onMapMessage;
    if (onDestinationSelectedValue != null) (_props$onMapMessage$o = (_props$onMapMessage = props.onMapMessage).onDestinationSelected) === null || _props$onMapMessage$o === void 0 ? void 0 : _props$onMapMessage$o.call(_props$onMapMessage, onDestinationSelectedValue);
  }, [onDestinationSelectedValue]);
  useEffect(() => {
    if (onNavigationPathGeneratedValue != null) {
      var _props$onMapMessage$o2, _props$onMapMessage2;
      let pathId, instructions, start, end, path;
      ({
        pathId,
        instructions,
        start,
        end,
        path
      } = onNavigationPathGeneratedValue);
      (_props$onMapMessage$o2 = (_props$onMapMessage2 = props.onMapMessage).onNavigationPathGenerated) === null || _props$onMapMessage$o2 === void 0 ? void 0 : _props$onMapMessage$o2.call(_props$onMapMessage2, pathId, instructions, start, end, path);
    }
  }, [onNavigationPathGeneratedValue]);
  useEffect(() => {
    var _props$onMapMessage$o3, _props$onMapMessage3;
    if (onUserPositionUpdatedValue != null) (_props$onMapMessage$o3 = (_props$onMapMessage3 = props.onMapMessage).onUserPositionUpdated) === null || _props$onMapMessage$o3 === void 0 ? void 0 : _props$onMapMessage$o3.call(_props$onMapMessage3, onUserPositionUpdatedValue);
  }, [onUserPositionUpdatedValue]);
  useEffect(() => {
    var _props$onMapMessage$o4, _props$onMapMessage4;
    if (onSearchDestinationsGeneratedValue != null) (_props$onMapMessage$o4 = (_props$onMapMessage4 = props.onMapMessage).onSearchDestinationsGenerated) === null || _props$onMapMessage$o4 === void 0 ? void 0 : _props$onMapMessage$o4.call(_props$onMapMessage4, onSearchDestinationsGeneratedValue);
  }, [onSearchDestinationsGeneratedValue]);
  useEffect(() => {
    var _props$onMapMessage$o5, _props$onMapMessage5;
    if (onMapTappedGenericValue != null) (_props$onMapMessage$o5 = (_props$onMapMessage5 = props.onMapMessage).onMapTappedGeneric) === null || _props$onMapMessage$o5 === void 0 ? void 0 : _props$onMapMessage$o5.call(_props$onMapMessage5, onMapTappedGenericValue);
  }, [onMapTappedGenericValue]);
  useImperativeHandle(parentRef, () => {
    const viewId = findNodeHandle(nodeRef.current);
    const listeners = {
      setMapVenue: credentials => {
        //, mapOptions: MAP_OPTIONS
        setTimeout(function () {
          setMapVenue({
            viewId,
            credentials
          });
        }, 500);
      },
      // setMapVenue: ({
      //   credentials,
      //   compass,
      //   floorSelector,
      //   zoomButtons,
      //   search,
      // }: {
      //   credentials: MAP_CREDENTIALS;
      //   compass?: MAP_COMPASS_PROPERTIES;
      //   floorSelector?: MAP_FLOOR_SELECTOR_PROPERTIES;
      //   zoomButtons?: MAP_ZOOM_BUTTON_PROPERTIES;
      //   search?: MAP_SEARCH_PROPERTIES;
      // }) => {
      //   setMapVenue({
      //     viewId,
      //     credentials,
      //     compass,
      //     floorSelector,
      //     zoomButtons,
      //     search,
      //   });
      // },
      centerMapOnUser: function (orientation) {
        let zoom = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;
        let animationDuration = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1000;
        centerMap(viewId, orientation, zoom, animationDuration);
      },
      setFollowBlueDot: function (enabled) {
        let rotate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
        let interval = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 500;
        setFollowBlueDot(viewId, enabled, rotate, interval);
      },
      setUserPosition: (lat, long, heading, confidence) => {
        setUserPosition(viewId, lat, long, heading, confidence, null);
      },
      setUserPositionXY: (x, y, heading, confidence) => {
        setUserPosition(viewId, x, y, heading, confidence, 'xy');
      },
      clearWayfindingPath: () => {
        clearWayfindingPath(viewId);
      },
      wayfindToWaypoint: waypointId => {
        wayfindToWaypoint(viewId, waypointId);
      },
      wayfindBetweenWaypoints: (originId, destinationId) => {
        wayfindBetweenWaypoints(viewId, originId, destinationId);
      },
      drawNavigationPath: function (pathId) {
        let shouldFocus = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
        let style = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
          stroke: '#ffc956',
          strokeWidth: 3
        };
        drawNavigationPath(viewId, pathId, shouldFocus, style);
      }
      //   styleUnitsForDestination: (
      //     destination: JIBESTREAM_DESTINATION,
      //     style: JIBESTREAM_STYLE = { fill: '#ABABAB' }
      //   ) => {
      //     styleUnitsForDestination(viewId, destination, style);
      //   },
      //   styleShape: (
      //     unitId: number,
      //     style: JIBESTREAM_STYLE = { fill: '#ABABAB' }
      //   ) => {
      //     styleShape(viewId, unitId, style);
      //   },
    };

    return listeners;
  });
  useEffect(() => {
    var _eventEmitter, _eventEmitter2, _eventEmitter3, _eventEmitter4, _eventEmitter5;
    console.log('Initializing Event Emitter');
    let eventEmitter = null;
    if (Platform.OS == 'android') {
      eventEmitter = DeviceEventEmitter;
    } else if (Platform.OS == 'ios') {
      console.log(CTPEventEmitter);
      eventEmitter = new NativeEventEmitter(CTPEventEmitter);
    }
    (_eventEmitter = eventEmitter) === null || _eventEmitter === void 0 ? void 0 : _eventEmitter.addListener('DESTINATION_SELECTED', data => setOnDestinationSelectedValue(data)
    // props.onMapMessage.onDestinationSelected?.(data)
    );

    (_eventEmitter2 = eventEmitter) === null || _eventEmitter2 === void 0 ? void 0 : _eventEmitter2.addListener('NAVIGATION_PATH_GENERATED', data => setOnNavigationPathGeneratedValue(data)
    // props.onMapMessage.onNavigationPathGenerated?.(
    //   data.pathId,
    //   data.instructions,
    //   data.start,
    //   data.end,
    //   data.path
    // )
    );

    (_eventEmitter3 = eventEmitter) === null || _eventEmitter3 === void 0 ? void 0 : _eventEmitter3.addListener('USER_POSITION_UPDATED', userPosition => setOnUserPositionUpdatedValue(userPosition)
    // props.onMapMessage.onUserPositionUpdated?.(userPosition)
    );

    (_eventEmitter4 = eventEmitter) === null || _eventEmitter4 === void 0 ? void 0 : _eventEmitter4.addListener('GENERATED_SEARCH_DESTINATIONS', destinations => setOnSearchDestinationsGeneratedValue(destinations)
    // props.onMapMessage.onSearchDestinationsGenerated?.(destinations)
    );

    // eventEmitter?.addListener('WINDOW_LOADED', () =>
    //   props.onMapMessage.onWindowLoaded?.()
    // );
    (_eventEmitter5 = eventEmitter) === null || _eventEmitter5 === void 0 ? void 0 : _eventEmitter5.addListener('MAP_TAPPED_GENERIC', event => setOnMapTappedGenericValue(event)
    // props.onMapMessage.onMapTappedGeneric?.(data)
    );
    // eventEmitter?.addListener('ODP_READY', (event: ODP_ON_READY) => {
    //   console.log('ODP onReady');
    //   props.onOdpEvent.onReady?.(event);
    // });
    // eventEmitter?.addListener(
    //   'ODP_KEY_VALIDATION',
    //   (event: ODP_ON_KEY_VALIDATION) => {
    //     console.log('ODP onKeyValidation');
    //     props.onOdpEvent.onKeyValidation?.(event);
    //   }
    // );
    // eventEmitter?.addListener(
    //   'ODP_NAVIGATION_UPDATE',
    //   (event: ODP_ON_NAVIGATION_UPDATE) => {
    //     props.onOdpEvent.onNavigationUpdate?.(event);
    //   }
    // );
    // eventEmitter?.addListener(
    //   'ODP_FACILITY_UPDATE',
    //   (event: ODP_ON_FACILITY_UPDATE) => {
    //     console.log('ODP onFacilityUpdate');
    //     props.onOdpEvent.onFacilityUpdate?.(event);
    //   }
    // );
    return () => {
      var _eventEmitter6, _eventEmitter7, _eventEmitter8;
      console.log('Removing Event Emitters');
      (_eventEmitter6 = eventEmitter) === null || _eventEmitter6 === void 0 ? void 0 : _eventEmitter6.removeAllListeners('DESTINATION_SELECTED');
      (_eventEmitter7 = eventEmitter) === null || _eventEmitter7 === void 0 ? void 0 : _eventEmitter7.removeAllListeners('NAVIGATION_PATH_GENERATED');
      (_eventEmitter8 = eventEmitter) === null || _eventEmitter8 === void 0 ? void 0 : _eventEmitter8.removeAllListeners('USER_POSITION_UPDATED');
      //   eventEmitter?.removeAllListeners('WINDOW_LOADED');
      //   eventEmitter?.removeAllListeners('MAP_TAPPED_GENERIC');
      //   eventEmitter?.removeAllListeners('ODP_READY');
      //   eventEmitter?.removeAllListeners('ODP_KEY_VALIDATION');
      //   eventEmitter?.removeAllListeners('ODP_NAVIGATION_UPDATE');
      //   eventEmitter?.removeAllListeners('ODP_FACILITY_UPDATE');
    };
  }, []);
  return /*#__PURE__*/React.createElement(SafeAreaView, {
    style: {
      flex: 1,
      width: '100%',
      height: '100%'
    }
  }, MapViewManager ? /*#__PURE__*/React.createElement(MapViewManager, {
    style: Platform.OS == 'android' ? {
      // converts dpi to px, provide desired height
      height: PixelRatio.getPixelSizeForLayoutSize(props.style.height),
      // converts dpi to px, provide desired width
      width: PixelRatio.getPixelSizeForLayoutSize(props.style.width)
    } : {
      height: props.style.height / 1,
      width: props.style.width / 1
      // converts dpi to px, provide desired height
      // height: PixelRatio.getPixelSizeForLayoutSize(props.style.height),
      // converts dpi to px, provide desired width
      // width: PixelRatio.getPixelSizeForLayoutSize(props.style.width),
    },

    ref: nodeRef
  }) : null);
};
const mapViewFwRef = /*#__PURE__*/forwardRef(MapView);
// mapViewFwRef.propTypes = {
//   eventEmitter: PropTypes.
// }
export default mapViewFwRef;
//# sourceMappingURL=MapView.js.map