import { BlockquoteHTMLAttributes } from 'react';

// {"id":425049,"status":true,"name":"Machine Shop","description":"","tags":[],"keywords":[],"extensors":{},"locations":[{"mapId":7216,"waypointIds":[43102987]}],"uris":{"items":[]},"displayMode":1,"_mappedPoints":{"7216":true}}
export type JIBESTREAM_DESTINATION = {
  id: number;
  name: string;
  description: string;
  tags: string[];
  keywords: string[];
  extensors: Object;
  locations: JIBESTREAM_LOCATION[];
  uris: Object;
  displayMode: number;
  waypoints: JIBESTREAM_WAYPOINT[];
};

export type JIBESTREAM_WAYPOINT = {
  id: number;
  mapId: number;
  coordinates: number[];
  associations: {
    destinations: number[];
  };
  xyz: {
    x: number;
    y: number;
    z: number;
  };
};

export type JIBESTREAM_PATH = {
  mapId: number;
  mover: {
    moverId: number;
    speed: number;
    accessiblity: number;
    typeName: string;
    svg: null;
  };
  points: {
    id: number;
    mapId: number;
    associations?: {
      destinations: number[];
    };
    xyz: {
      x: number;
      y: number;
      z: number;
    };
    x: number;
    y: number;
    z: number;
  }[];
  cost: number;
  pixelDistance: number;
  mmDistance: number;
};

export type JIBESTREAM_LOCATION = {
  mapId: number;
  waypointIds: number[];
};

export type JIBESTREAM_STYLE = {
  fill?: string;
  stroke?: string;
  strokeWidth?: number;
  strokeOpacity?: number;
  opacity?: number;
};

export type JIBESTREAM_FONT = {
  align?: string;
  fill?: string;
  fontFamily?: string;
  fontSize?: number & string;
  wordWrap?: boolean;
  wordWrapWidth?: number;
};

export type INSTRUCTION = {
  fullText: string;
  prepend: string;
  direction: string;
  destinationName?: string;
  distance: number;
  remainingDistance: number;
  wp: number;
  dest: number;
  startPoint?: number[];
  endPoint?: number[];
};

export type MAP_CREDENTIALS = {
  clientId: string;
  clientSecret: string;
  customerId: number;
  venueId: number;
  host: string;
};

export type MAP_OPTIONS = {
  snapToGrid?: boolean;
  snapToWaypoint?: boolean;
  snapToNavigationPath?: boolean;
  snapToNavigationPathThreshold?: number;
  userSpeed?: number;
  autoReroute?: boolean;
  autoRerouteThreshold?: number;
};

export type MAP_FLOOR_SELECTOR_PROPERTIES = {
  inactiveStyle?: JIBESTREAM_STYLE;
  inactiveFont?: JIBESTREAM_FONT;
  activeStyle?: JIBESTREAM_STYLE;
  activeFont?: JIBESTREAM_FONT;
  position?: number[];
  padding?: number[];
  vertical?: boolean;
};

export type MAP_SEARCH_PROPERTIES = {
  inactiveStyle?: JIBESTREAM_STYLE;
  inactiveFont?: JIBESTREAM_FONT;
  activeStyle?: JIBESTREAM_STYLE;
  activeFont?: JIBESTREAM_FONT;
  position?: number[];
  placeholder?: string;
  searchArray?: Object[];
  maxResults?: number;
  padding?: number[];
};

export type MAP_ZOOM_BUTTON_PROPERTIES = {
  inactiveStyle?: JIBESTREAM_STYLE;
  inactiveFont?: JIBESTREAM_FONT;
  activeStyle?: JIBESTREAM_STYLE;
  activeFont?: JIBESTREAM_FONT;
  position?: number[];
  zoomInText?: string;
  zoomOutText?: string;
  padding?: number[];
};

export type FOLLOW_BLUE_DOT_PROPERTIES = {
  enabled: boolean;
  rotate: boolean;
  interval: number;
};

export type MAP_TAP_EVENT = {
  x: number;
  y: number;
};
